/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

/**
 * A Notification for AttributeChange sent by <code>MBeans</code>.
 *
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.4 $
 */
public class AttributeChangeNotification extends Notification
{
	private static final long serialVersionUID = 535176054565814134L;

    /** 
     * A Notification type for an attribute change
     */
	public static final String ATTRIBUTE_CHANGE = "jmx.attribute.change";

	private String attributeName;
	private String attributeType;
	private Object oldValue;
	private Object newValue;

    /**
     * Creates an <code>AttributeChangeNotification</code>
     *
     * @param source The source of this notification
     * @param sequenceNumber The sequence number of this notification
     * @param timestamp The timestamp of this notification
     * @param message The message for the notification
     * @param attributeName The name of the changed attribute
     * @param attributeType The type of the attribute   
     * @param oldValue The old value of the Attribute
     * @param newValue The new value of the Attribute
     *
     */
	public AttributeChangeNotification(Object source, long sequenceNumber, long timestamp, String message, String attributeName, String attributeType, Object oldValue, Object newValue)
    {
		super(ATTRIBUTE_CHANGE, source, sequenceNumber, timestamp, message);
		this.attributeName = attributeName;
		this.attributeType = attributeType;
		this.oldValue = oldValue;
		this.newValue = newValue;
    }

    /**
     * Returns the attribute name of this change notification.
     *
     * @return String Returns the attribute name
     *
     */
	public String getAttributeName() {return attributeName;}

    /**
     * Returns the attribute type of this change notification.
     *
     * @return String The attribute type
     *
     */
	public String getAttributeType() {return attributeType;}


    /**
     * Returns the old value of the attribute
     *
     * @return Object The old value
     */
	public Object getOldValue() {return oldValue;}
    
    /**
     * Returns the new value of the attribute
     *
     * @return Object The new  value
     */
	public Object getNewValue() {return newValue;}
}
