/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

/**
 * Defines methods that should be implemented by a <code>Dynamic MBean</code>
 *
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.3 $
 */
public interface DynamicMBean
{
    /**
     * Provides the exposed attributes/actions of this <code>DynamicMBean</code>
     *
     * @return MBeanInfo an instance of <code>MBeanInfo</code>
     */
	public MBeanInfo getMBeanInfo();
    
    /**
     * Returns the value of the given attribute.
     *
     * @param attribute The attribute name for the value being retrieved.
     * @return Object The value of the attribute.
     * @throws AttributeNotFoundException If the given attribute isn't found.
     * @throws MBeanException Wraps a java.lang.Exception
     * @throws ReflectionException Wraps a java.lang.Exception while trying
     *           to invoke the getter.
     *
     */
	public Object getAttribute(String attribute) throws AttributeNotFoundException, MBeanException, ReflectionException;
    
    /**
     * Set the value of the specific attribute
     *
     * @param attribute The <code>Attribute</code> being set.
     * @throws AttributeNotFoundException 
     * @throws InvalidAttributeValueException
     * @throws MBeanException Wraps a java.lang.Exception which is thrown
     *              by the setter.
     * 
     */
	public void setAttribute(Attribute attribute) throws AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException;

    /**
     * Returns an <code>AttributeList</code> given an array of attributes.
     * 
     * @param attributes The array of attributes being retrieved
     * @return AttributeList The list of <code>Attribute</code>
     * 
     */
	public AttributeList getAttributes(String[] attributes);

    /**
     * Set the attribute using the given attribute as an <code>AttributeList</code>
     * 
     * @param attributes The attribute being set.
     * @return AttributeList the list of <code>Attribute</code> successfully set.
     */
	public AttributeList setAttributes(AttributeList attributes);


    /**
     * Invoke the given method.
     *
     * @param method The name of the method to be invoked.
     * @param arguments The array of arguments to be set.
     * @param params Contains the signature of the method to be invoked.
     * @return Object The object returned by the invocaton.
     * @throws MBeanException Wraps a java.lang.Exception 
     * @throws ReflectionException
     * 
     */
	public Object invoke(String method, Object[] arguments, String[] params) throws MBeanException, ReflectionException;
}
