/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import java.lang.reflect.Method;

import mx4j.util.Utils;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.4 $
 */
public class MBeanAttributeInfo extends MBeanFeatureInfo
{
	private static final long serialVersionUID = 8644704819898565848L;

	private String attributeType;
	private boolean isRead;
	private boolean isWrite;
	private boolean is;

	public MBeanAttributeInfo(String name, String description, Method getter, Method setter) throws IntrospectionException
	{
		super(name, description);

		if (getter == null && setter == null) {throw new IntrospectionException("Attribute must be readable or writable");}

		if (getter != null)
		{
			if (Utils.isAttributeGetter(getter))
			{
				this.isRead = true;
				if (getter.getName().startsWith("is")) {this.is = true;}
				this.attributeType = getter.getReturnType().getName();
			}
			else {throw new IntrospectionException("Bad getter method");}
		}
		if (setter != null)
		{
			if (Utils.isAttributeSetter(setter))
			{
				this.isWrite = true;
				this.attributeType = setter.getParameterTypes()[0].getName();
			}
			else {throw new IntrospectionException("Bad setter method");}
		}
	}

	public MBeanAttributeInfo(String name, String className, String description, boolean isReadable, boolean isWritable, boolean isIs)
	{
		super(name, description);

		if (!isReadable && !isWritable) {throw new RuntimeOperationsException(new IllegalArgumentException("Attribute must be readable or writable"));}

		this.attributeType = className == null ? "" : className;
		this.isRead = isReadable;
		this.isWrite = isWritable;
		this.is = isIs;
	}

	public String getType() {return attributeType;}
	public boolean isReadable() {return isRead;}
	public boolean isWritable() {return isWrite;}
	public boolean isIs() {return is;}

	public int hashCode()
	{
		int hash = super.hashCode();

		String type = getType();
		hash ^= type == null ? 0 : type.hashCode();

		hash ^= isReadable() ? Boolean.TRUE.hashCode() : Boolean.FALSE.hashCode();
		hash ^= isWritable() ? Boolean.TRUE.hashCode() : Boolean.FALSE.hashCode();
		hash ^= isIs() ?  Boolean.TRUE.hashCode() : Boolean.FALSE.hashCode();

		return hash;
	}

	public boolean equals(Object obj)
	{
		if (!super.equals(obj)) {return false;}

		try
		{
			MBeanAttributeInfo other = (MBeanAttributeInfo)obj;

			String type = getType();
			String otherType = other.getType();
			if (type != null && !type.equals(otherType)) {return false;}
			if (type == null && otherType != null) {return false;}

			if (isReadable() ^ other.isReadable()) {return false;}
			if (isWritable() ^ other.isWritable()) {return false;}
			if (isIs() ^ other.isIs()) {return false;}

			return true;
		}
		catch (ClassCastException ignored) {}
		return false;
	}
}
