/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import java.lang.reflect.Method;

import mx4j.util.Utils;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.5 $
 */
public class MBeanOperationInfo extends MBeanFeatureInfo
{
	private static final long serialVersionUID = -6178860474881375330L;

	public static final int INFO = 0;
	public static final int ACTION = 1;
	public static final int ACTION_INFO = 2;
	public static final int UNKNOWN = 3;

	private MBeanParameterInfo[] signature;
	private String type;
	private int impact;

	public MBeanOperationInfo(String description, Method method)
	{
		super(method.getName(), description);
		Class[] params = method.getParameterTypes();
		this.signature = new MBeanParameterInfo[params.length];
		for (int i = 0; i < params.length; ++i)
		{
			this.signature[i] = new MBeanParameterInfo("", params[i].getName(), "");
		}
		this.type = method.getReturnType().getName();
		this.impact = UNKNOWN;
	}
	public MBeanOperationInfo(String name, String description, MBeanParameterInfo[] signature, String type, int impact)
	{
		super(name, description);
		this.signature = signature == null ? new MBeanParameterInfo[0] : signature;
		this.type = type;
		this.impact = impact;
	}

    public String getReturnType() {return type;}

	public MBeanParameterInfo[] getSignature() {return signature;}

	public int getImpact() {return impact;}

	public int hashCode()
	{
		int hash = super.hashCode();

		String type = getReturnType();
		hash ^= type == null ? 0 : type.hashCode();

		hash ^= Utils.arrayHashCode(getSignature());

		return hash;
	}

	public boolean equals(Object obj)
	{
		if (!super.equals(obj)) {return false;}

		try
		{
			MBeanOperationInfo other = (MBeanOperationInfo)obj;

			String type = getReturnType();
			String otherType = other.getReturnType();
			if (type != null && !type.equals(otherType)) {return false;}
			if (type == null && otherType != null) {return false;}

			// No checks on description and impact

			return Utils.arrayEquals(getSignature(), other.getSignature());
		}
		catch (ClassCastException ignored) {}
		return false;
	}
}
