/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

/**
 * Implemented by an MBean that need to carry out operations before and after
 * being registered or de-registered.
 * 
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.3 $
 */
public interface MBeanRegistration
{

    /**
     * Allows the MBean to perform any operations before being registered
     * to the <code>MBeanServer</code>.
     *
     * <p>
     * Any exception will cause the MBean not being registered. 
     * </p>
     *
     * @param server The MBeanServer on which the MBean will be registered.
     * @param name The <code>ObjectName</code> of the MBean.
     * @return ObjectName The name of the registered MBean
     * @throws Exception Exception of the operation. Note that this is caught
     *          by the <code>MBeanServer</code> and re-thrown as an 
     *          <code>MBeanRegistrationException</code>.
     *
     */
	public ObjectName preRegister(MBeanServer server, ObjectName name) throws Exception;

    /**
     * Called upon after a registration ( successful or not ).
     * 
     * @param registrationDone Evaluates to true of the registrations 
     *                  is successful, false otherwise.
     *
     */
	public void postRegister(Boolean registrationDone);

    /**
     * Called upon before an <code>MBean</code> will  be de-registered by   
     * the <code>MBeanServer</code>.
     *
     * @throws Exception Would be caught by the <code>MBeanServer</code>
     *                  and rethrown as an MBeanRegistrationException.
     */
	public void preDeregister() throws Exception;

    /**
     * Called upon after the MBean has been de-registered.
     *
     * 
     */
	public void postDeregister();
}
