/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import java.io.ObjectInputStream;
import java.util.Set;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.3 $
 */
public interface MBeanServer
{
	public void addNotificationListener(ObjectName observed, NotificationListener listener, NotificationFilter filter, Object handback)
			throws InstanceNotFoundException;

	public void addNotificationListener(ObjectName observed, ObjectName listener, NotificationFilter filter, Object handback)
			throws InstanceNotFoundException;

	public void removeNotificationListener(ObjectName observed, NotificationListener listener)
			throws InstanceNotFoundException, ListenerNotFoundException;

	public void removeNotificationListener(ObjectName observed, ObjectName listener)
			throws InstanceNotFoundException, ListenerNotFoundException;
/*
	// Not present in the specification but needed
	public void removeNotificationListener(ObjectName observed, ObjectName listener, NotificationFilter filter, Object handback)
			throws InstanceNotFoundException, ListenerNotFoundException;

	// Not present in the specification but needed
	public void removeNotificationListener(ObjectName observed, NotificationListener listener, NotificationFilter filter, Object handback)
			throws InstanceNotFoundException, ListenerNotFoundException;
*/
	public Object instantiate(String className)
			throws ReflectionException, MBeanException;

	public Object instantiate(String className, ObjectName loaderName)
			throws ReflectionException, MBeanException, InstanceNotFoundException;

	public Object instantiate(String className, ObjectName loaderName, Object[] args, String[] parameters)
			throws ReflectionException, MBeanException, InstanceNotFoundException;

	public Object instantiate(String className, Object[] args, String[] parameters)
			throws ReflectionException, MBeanException;

	public ObjectInstance createMBean(String className, ObjectName objectName)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException;

	public ObjectInstance createMBean(String className, ObjectName objectName, ObjectName loaderName)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException;

	public ObjectInstance createMBean(String className, ObjectName objectName, Object[] args, String[] parameters)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException;

	public ObjectInstance createMBean(String className, ObjectName objectName, ObjectName loaderName, Object[] args, String[] parameters)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException;

	public ObjectInstance registerMBean(Object mbean, ObjectName objectName)
			throws InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException;

	public void unregisterMBean(ObjectName objectName)
			throws InstanceNotFoundException, MBeanRegistrationException;

	public ObjectInputStream deserialize(String className, ObjectName loaderName, byte[] bytes)
			throws InstanceNotFoundException, OperationsException, ReflectionException;

	public ObjectInputStream deserialize(String className, byte[] bytes)
			throws OperationsException, ReflectionException;

	public ObjectInputStream deserialize(ObjectName objectName, byte[] bytes)
			throws InstanceNotFoundException, OperationsException;

	public Object getAttribute(ObjectName objectName, String attribute)
			throws MBeanException, AttributeNotFoundException, InstanceNotFoundException, ReflectionException;

	public void setAttribute(ObjectName objectName, Attribute attribute)
			throws InstanceNotFoundException, AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException;

	public AttributeList getAttributes(ObjectName objectName, String[] attributes)
			throws InstanceNotFoundException, ReflectionException;

	public AttributeList setAttributes(ObjectName objectName, AttributeList attributes)
			throws InstanceNotFoundException, ReflectionException;

	public Object invoke(ObjectName objectName, String methodName, Object[] args, String[] parameters)
			throws InstanceNotFoundException, MBeanException, ReflectionException;

	public String getDefaultDomain();

	public Integer getMBeanCount();

	public boolean isRegistered(ObjectName objectname);

	public MBeanInfo getMBeanInfo(ObjectName objectName)
			throws InstanceNotFoundException, IntrospectionException, ReflectionException;

	public ObjectInstance getObjectInstance(ObjectName objectName)
			throws InstanceNotFoundException;

	public boolean isInstanceOf(ObjectName objectName, String className)
			throws InstanceNotFoundException;

	public Set queryMBeans(ObjectName patternName, QueryExp filter);

	public Set queryNames(ObjectName patternName, QueryExp filter);
}
