/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.9 $
 */
public class MBeanServerDelegate extends NotificationBroadcasterSupport implements MBeanServerDelegateMBean
{
	private String m_mbeanServerId;
	private static long m_mbeanServerCount;
	// Only notifications of type MBeanServerNotification are emitted by this class, so the array must have length 1
	private static final MBeanNotificationInfo[] m_notifications = new MBeanNotificationInfo[] {new MBeanNotificationInfo
	(
		new String[] {MBeanServerNotification.REGISTRATION_NOTIFICATION, MBeanServerNotification.UNREGISTRATION_NOTIFICATION},
		MBeanServerNotification.class.getName(), // as required by the spec
		"Notifications emitted by the MBeanServerDelegate MBean upon registration or unregistration of MBeans"
	)};

	public MBeanServerDelegate()
	{
		String host = "localhost";
		try {host = InetAddress.getLocalHost().getHostName();}
		catch (UnknownHostException ignored) {}
		synchronized (MBeanServerDelegate.class) {++m_mbeanServerCount;}
		m_mbeanServerId = host + "_" + m_mbeanServerCount;
	}

	public String getMBeanServerId() {return m_mbeanServerId;}
	public String getImplementationName() {return "MX4J";}
	public String getImplementationVendor() {return "MX4J";}
	// TODO: read it from the manifest
	public String getImplementationVersion() {return "1.1.1";}
	public String getSpecificationName() {return "Java Management Extensions";}
	public String getSpecificationVendor() {return "Sun Microsystem";}
	public String getSpecificationVersion() {return "1.1 Final Release";}

	public MBeanNotificationInfo[] getNotificationInfo() {return m_notifications;}
}
