/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

/**
 * Defines the management interface for a MBeanServerDelegate
 * 
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.3 $
 */
public interface MBeanServerDelegateMBean
{
    /**
     * Returns the implementation name, e.g. MX4J.
     *
     * @return String The implementation name.
     * 
     */
	public String getImplementationName();
    
    /**
     * Returns the implementation vendor, e.g. MX4J.
     *
     * @return String The implementation vendor.
     * 
     */
	public String getImplementationVendor();
    
    /**
     * Returns the implementation version, e.g. 1.1.
     *
     * @return String The implementation version.
     * 
     */
	public String getImplementationVersion();

    /**
     * Returns the <code>MBeanServer</code> id.
     * 
     * @return String The MBeanServer id.
     */
	public String getMBeanServerId();

    /**
     * Returns the specification name.
     *
     * @return String The specification name (e.g. Java Management Extensions).
     */
	public String getSpecificationName();

    /**
     * Returns the specification vendor
     *
     * @return String The specifcation vendor
     *
     */
	public String getSpecificationVendor();
    
    /**
     * Returns the specification version
     * 
     * @return String The specification version
     */
	public String getSpecificationVersion();
}
