/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.EventObject;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.5 $
 */
public class Notification extends EventObject
{
	// Support only for JMX 1.1 serialization
    private static final long serialVersionUID = -7516092053498031989L;

	private String type;
	private long sequenceNumber;
	private long timeStamp;
	private String message;
	private Object userData;
	/*
	 * A duplicate for the existing data member in EventObject: this one is not transient and should hold only
	 * ObjectNames (so they are Serializable and can be sent along the wire for remote notifications), and it's
	 * protected and not private for a mistake in JMX 1.0, but for binary compatibility we leave it protected.
	 */
	protected Object source;

	public Notification(String type, Object source, long sequenceNumber) {this(type, source, sequenceNumber, System.currentTimeMillis(), null);}
	public Notification(String type, Object source, long sequenceNumber, long timeStamp) {this(type, source, sequenceNumber, timeStamp, null);}
	public Notification(String type, Object source, long sequenceNumber, String message) {this(type, source, sequenceNumber, System.currentTimeMillis(), message);}
	public Notification(String type, Object source, long sequenceNumber, long timeStamp, String message)
	{
		// Data member source of EventObject is transient
		super(source);
		this.source = source;
		this.type = type;
		this.sequenceNumber = sequenceNumber;
		this.timeStamp = timeStamp;
		this.message = message;
	}

	public String getMessage() {return message;}
	public long getSequenceNumber() {return sequenceNumber;}
	public long getTimeStamp() {return timeStamp;}
	public String getType() {return type;}
	public Object getUserData() {return userData;}
	public void setUserData(Object userData) {this.userData = userData;}
	public void setSequenceNumber(long sequenceNumber) {this.sequenceNumber = sequenceNumber;}
	public void setSource(Object source) {this.source = source;}
	public void setTimeStamp(long timeStamp) {this.timeStamp = timeStamp;}
	public Object getSource() {return this.source;}

	public String toString()
	{
		StringBuffer b = new StringBuffer("[");
		b.append("source=").append(getSource()).append(", ");
		b.append("message=").append(getMessage()).append(", ");
		b.append("sequence=").append(getSequenceNumber()).append(", ");
		b.append("type=").append(getType()).append(", ");
		b.append("time=").append(getTimeStamp()).append(", ");
		b.append("data=").append(getUserData());
		b.append("]");
		return b.toString();
	}

	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException
	{
		in.defaultReadObject();
		super.source = source;
	}
}
