/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

/**
 * Implemented by an <code>MBean</code> that emits <code>Notification</code>.
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.4 $
 */
public interface NotificationBroadcaster
{

    /**
     * Returns the name of the java class of the notification and the 
     * notification types sent. 
     *
     * @return MBeanNotificationInfo The NotificationInfo
     */
	public MBeanNotificationInfo[] getNotificationInfo();


    /**
     * Adds a listener to the registered MBean.
     *
     * @param listener The listener that will handle the notifications emitted
     *                  by the registered MBean.
     * @param filter The <code>NotificationFilter</code> to use.
     * @param handback The object to be sent back to the listener.
     * @throws IllegalArgumentException If listener is null.
     *
     */
	public void addNotificationListener(NotificationListener listener, NotificationFilter filter, Object handback);
	// This method is not be part of the spec, but is present in the RI (probably a bug)
    /**
     * Removes a listener. All entries corresponding to the listener will be 
     * removed (in the case of the listener being registered with different
     * handbacks and filters ).
     *
     * @param listener The <code>NotificationListener</code>
     * @throws ListenerNotFoundException If no such listener exists.
     *
     *
     */
	public void removeNotificationListener(NotificationListener listener) throws ListenerNotFoundException;
	// This method is part of the spec, but is not present in the RI (probably a bug)
//	public void removeNotificationListener(NotificationListener listener, Object handback) throws ListenerNotFoundException;
	// This method is not part of the spec nor of the RI, but it is needed
//	public void removeNotificationListener(NotificationListener listener, NotificationFilter filter, Object handback) throws ListenerNotFoundException;
}
