/**
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import java.util.Vector;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.io.ObjectStreamField;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.6 $
 */
public class NotificationFilterSupport implements NotificationFilter
{
	private static final long serialVersionUID = 6579080007561786969L;
	private static final String serialName = "enabledTypes";
	private static final ObjectStreamField[] serialPersistentFields =
	{
		new ObjectStreamField(serialName, List.class)
	};

	private HashSet m_types = new HashSet();

	public void enableType(String prefix) throws IllegalArgumentException
	{
		if (prefix == null)
		{
            throw new IllegalArgumentException("Null notification type");
		}
		m_types.add(prefix);
	}

	public void disableAllTypes()
	{
		m_types.clear();
	}

	public void disableType(String prefix)
	{
        m_types.remove(prefix);
	}

	public Vector getEnabledTypes()
	{
		Vector v = new Vector();
		v.addAll(m_types);
		return v;
	}

	public boolean isNotificationEnabled(Notification notification)
	{
		String type = notification.getType();
		if (type != null)
		{
            for (Iterator i = m_types.iterator(); i.hasNext();)
			{
				String t = (String)i.next();
                if (type.startsWith(t))
				{
					return true;
				}
			}
		}
		return false;
	}

	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException
	{
		ObjectInputStream.GetField fields = in.readFields();

		Vector vector = (Vector)fields.get(serialName, null);
		if (fields.defaulted(serialName))
		{
            throw new IOException("Serialized stream corrupted: expecting a non-null Vector");
		}

		if (m_types == null) {m_types = new HashSet();}
		m_types.clear();
		m_types.addAll(vector);
	}

    private void writeObject(ObjectOutputStream out) throws IOException
	{
		ObjectOutputStream.PutField fields = out.putFields();

		Vector vector = getEnabledTypes();
		fields.put(serialName, vector);
		out.writeFields();
	}
}
