/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import javax.management.MBeanException;
import javax.management.RuntimeOperationsException;
import javax.management.InstanceNotFoundException;

/**
 * Implements by an MBean that are meant to be persistent.
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.3 $
 */
public interface PersistentMBean
{

    /**
     * Instantiates for the data found in the persistance store.
     *
     * @throws MBeanException Wraps another exception or thrown when
     *                  persistance is not supported.
     * @throws RuntimeOperationsException Wraps exception from persistance operation
     * @throws InstanceNotFoundException Cant find the <code>MBean</code> from
     *          the persistance store. 
     */
	public void load() throws MBeanException, RuntimeOperationsException, InstanceNotFoundException;
    
    /**
     * Captures the current state of the MBean.
     *
     * 
     * @throws MBeanException
     * @throws RuntimeOperationsException
     * @throws InstanceNotFoundException 
     *
     */
	public void store() throws MBeanException, RuntimeOperationsException, InstanceNotFoundException;
}
