/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.4 $
 */
public class ReflectionException extends JMException
{
	private static final long serialVersionUID = 9170809325636915553L;

	private Exception exception;

	public ReflectionException(Exception x)
	{
		this.exception = x;
	}

	public ReflectionException(Exception x, String message)
	{
		super(message);
		this.exception = x;
	}

	public Exception getTargetException() {return exception;}

	public String getMessage()
	{
		String msg = "";
		String m1 = super.getMessage();
		if (m1 != null) {msg += m1;}
		if (exception != null)
		{
			String m2 = exception.toString();
			if (m2 != null) {msg += "nested exception is " + m2;}
		}
		return msg;
	}

	public void printStackTrace()
	{
		if (exception == null) {super.printStackTrace();}
		else
		{
			synchronized (System.err)
			{
				System.err.println(this);
				exception.printStackTrace();
			}
		}
	}

	public void printStackTrace(PrintStream s)
	{
		if (exception == null) {super.printStackTrace(s);}
		else
		{
			synchronized (s)
			{
				s.println(this);
				exception.printStackTrace(s);
			}
		}
	}

	public void printStackTrace(PrintWriter w)
	{
		if (exception == null) {super.printStackTrace(w);}
		else
		{
			synchronized (w)
			{
				w.println(this);
				exception.printStackTrace(w);
			}
		}
	}
}
