/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.3 $
 */
public class RuntimeErrorException extends JMRuntimeException
{
	private static final long serialVersionUID = 704338937753949796L;

	private Error error;

	public RuntimeErrorException(Error error)
	{
		this.error = error;
	}

	public RuntimeErrorException(Error error, String message)
	{
		super(message);
		this.error = error;
	}

	public Error getTargetError() {return error;}

	public String getMessage()
	{
		String msg = "";
		String m1 = super.getMessage();
		if (m1 != null) {msg += m1;}
		if (error != null)
		{
			String m2 = error.toString();
			if (m2 != null) {msg += "nested error is " + m2;}
		}
		return msg;
	}

	public void printStackTrace()
	{
		if (error == null) {super.printStackTrace();}
		else
		{
			synchronized (System.err)
			{
				System.err.println(this);
				error.printStackTrace();
			}
		}
	}

	public void printStackTrace(PrintStream s)
	{
		if (error == null) {super.printStackTrace(s);}
		else
		{
			synchronized (s)
			{
				s.println(this);
				error.printStackTrace(s);
			}
		}
	}

	public void printStackTrace(PrintWriter w)
	{
		if (error == null) {super.printStackTrace(w);}
		else
		{
			synchronized (w)
			{
				w.println(this);
				error.printStackTrace(w);
			}
		}
	}
}
