/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management;

import java.io.PrintWriter;
import java.io.PrintStream;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.4 $
 */
public class RuntimeMBeanException extends JMRuntimeException
{
	private static final long serialVersionUID = 5274912751982730171L;

	private RuntimeException runtimeException;

	public RuntimeMBeanException(RuntimeException exception)
	{
		this.runtimeException = exception;
	}

	public RuntimeMBeanException(RuntimeException exception, String message)
	{
		super(message);
		this.runtimeException = exception;
	}

	public RuntimeException getTargetException() {return runtimeException;}

	public String getMessage()
	{
		String msg = "";
		String m1 = super.getMessage();
		if (m1 != null) {msg += m1;}
		if (runtimeException != null)
		{
			String m2 = runtimeException.toString();
			if (m2 != null) {msg += "nested runtime exception is " + m2;}
		}
		return msg;
	}

	public void printStackTrace()
	{
		if (runtimeException == null) {super.printStackTrace();}
		else
		{
			synchronized (System.err)
			{
				System.err.println(this);
				runtimeException.printStackTrace();
			}
		}
	}

	public void printStackTrace(PrintStream s)
	{
		if (runtimeException == null) {super.printStackTrace(s);}
		else
		{
			synchronized (s)
			{
				s.println(this);
				runtimeException.printStackTrace(s);
			}
		}
	}

	public void printStackTrace(PrintWriter w)
	{
		if (runtimeException == null) {super.printStackTrace(w);}
		else
		{
			synchronized (w)
			{
				w.println(this);
				runtimeException.printStackTrace(w);
			}
		}
	}

}
