/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management.loading;

import javax.management.MBeanServerFactory;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.4 $
 */

/**
 * A Singleton class which keeps the list of ClassLoaders registered in the MBeanServer. It provides methods to load classes using the registered classLoaders
 */
public class DefaultLoaderRepository
{
	// A singleton
	private DefaultLoaderRepository() {}

	/**
	 * <p> Go through the list of class loaders and try to load the requested class. The method will stop as soon as the class is found,
	 * if it is not found a ClassNotFoundException will be thrown</p>
	 * @see javax.management.MBeanServerFactory#getClassLoaderRepository
	 *
	 * @param className - the string represented name of the class to be loaded.
	 * @return the found class
	 * @throws ClassNotFoundException - if the class was not found
	 */
	public static Class loadClass(String className) throws ClassNotFoundException
	{
		return MBeanServerFactory.getClassLoaderRepository(null).loadClass(className);
	}

	/**
	 * <p> Try to load the class with the list of classloaders without using the given classLoader. The method will stop as soon as the class is found,
	 * will throw a ClassNotFoundException if the class is not found</p>
	 *
	 * @param loader - the ClassLoader to <b>exclude</b> from the list of classLoaders used to load the class
	 * @paran className - the string represented name of the class to load
	 * @return - the found class
	 * @throws ClassNotFoundException - if the class was not found.
	 */
	public static Class loadClassWithout(ClassLoader loader, String className) throws ClassNotFoundException
	{
		return MBeanServerFactory.getClassLoaderRepository(null).loadClassWithout(loader, className);
	}
}
