/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management.loading;

import java.util.Enumeration;
import java.util.Set;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;

import javax.management.ServiceNotFoundException;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.3 $
 */
public interface MLetMBean
{
	public void addURL(String url) throws ServiceNotFoundException;
	public void addURL(URL url);
	public URL[] getURLs();

	public String getLibraryDirectory();
	public void setLibraryDirectory(String libdir);

	/**
	 * <p>Finds the resource with the given name. A resource is some data (images, audio, text, etc) that can be accessed by class code in a way that is independent of the location of the code.
	 * The name of a resource is a "/"-separated path name that identifies the resource </p>
	 *
	 * @param name - The resource name
	 * @return An URL for reading the resource, or null if the resource could not be found or the caller doesn't have adequate privileges to get the resource
	 */
	public URL getResource(String name);

	/**
	 * <p>Returns an input stream for reading the specified resource. The search order is described in the documentation for getResource(String).</p>
	 *
	 * @param name - The resource name
	 * @return An input stream for reading the resource, or null if the resource could not be found.
	 */
	public InputStream getResourceAsStream(String name);

	/**
	 * <p>Finds all the resources with the given name. A resource is some data (images, audio, text, etc) that can be accessed by class code in a way that is independent of the location of the code.
	 * The name of a resource is a "/"-separated path name that identifies the resource</p>
	 *
	 * @param name - The resource name
	 * @return An enumeration of URL to the resource. If no resources could be found, the enumeration will be empty. Resources that the doesn't have access to will not be in the enumeration.
	 * @throws IOException
	 */
	public Enumeration getResources(String name) throws IOException;

	public Set getMBeansFromURL(String url) throws ServiceNotFoundException;
	public Set getMBeansFromURL(URL url) throws ServiceNotFoundException;
}
