/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package javax.management.monitor;

import javax.management.MBeanNotificationInfo;

/**
 *
 *
 * @see Monitor
 * @author <a href="mailto:tibu@users.sourceforge.net">Carlos Quiroz</a>
 * @version $Revision: 1.3 $
 */
public class StringMonitor extends Monitor implements StringMonitorMBean
{
	private String stringToCompare;

	private boolean notifyMatch, notifyDiffers;

	private String derivedGauge;

	private long derivedGaugeTimesamp;

	private transient String lastValue;

	private static final MBeanNotificationInfo[] notificationInfos =
	{
		new MBeanNotificationInfo(new String[]
			{
				MonitorNotification.RUNTIME_ERROR,
				MonitorNotification.OBSERVED_OBJECT_ERROR,
				MonitorNotification.OBSERVED_ATTRIBUTE_ERROR,
				MonitorNotification.OBSERVED_ATTRIBUTE_TYPE_ERROR,
				MonitorNotification.STRING_TO_COMPARE_VALUE_MATCHED,
				MonitorNotification.STRING_TO_COMPARE_VALUE_DIFFERED
			}
		, "javax.management.monitor.MonitorNotification", "Notifications sent by the StringMonitor MBean")
	};

	public synchronized void start()
	{
		doStart();
	}

	public synchronized void stop() {
		doStop();
	}

	synchronized void executeMonitor(Object attributeValue)
	{
		if (lastValue != null && lastValue.equals(attributeValue))
		{
			return;
		}
		if (stringToCompare == null)
		{
			getLogger().info(new StringBuffer("Monitor ").append(this).append(" target value is null"));
			notifyListeners("jmx.notification.error.type", objectName, attribute);
		}
		if (!(attributeValue instanceof String))
		{
			getLogger().info(new StringBuffer("Monitor ").append(this).append(" attribute is not String"));
			notifyListeners("jmx.notification.error.type", objectName, attribute);
			return;
		}
		calculateDerivedGauge((String)attributeValue);
		boolean matches = attributeValue.equals(stringToCompare);
		lastValue =(String) attributeValue;
		if (notifyMatch && matches)
		{
			getLogger().info(new StringBuffer("Monitor ").append(this).append(" found a match"));
			notifyListeners("jmx.monitor.string.matches", objectName, attribute);
		}
		if (notifyDiffers && !matches)
		{
			getLogger().info(new StringBuffer("Monitor ").append(this).append(" found a difference"));
			notifyListeners("jmx.monitor.string.differs", objectName, attribute);
		}
	}

	void calculateDerivedGauge(String value)
	{
		derivedGauge = value;
		derivedGaugeTimesamp = System.currentTimeMillis();
	}

	public String getDerivedGauge()
	{
		return derivedGauge;
	}

	public long getDerivedGaugeTimeStamp()
	{
		return derivedGaugeTimesamp;
	}

	public String getStringToCompare()
	{
		return stringToCompare;
	}

	public void setStringToCompare(String value) throws java.lang.IllegalArgumentException
	{
		if (value == null)
		{
			throw new IllegalArgumentException("Cannot compare to null");
		}
		lastValue = null;
		this.stringToCompare = value;
	}

	public boolean getNotifyMatch()
	{
		return notifyMatch;
	}

	public synchronized void setNotifyMatch(boolean value)
	{
		lastValue = null;
		this.notifyMatch = value;
	}

	public boolean getNotifyDiffer()
	{
		return notifyDiffers;
	}

	public synchronized void setNotifyDiffer(boolean value)
	{
		lastValue = null;
		this.notifyDiffers = value;
	}

	public MBeanNotificationInfo[] getNotificationInfo()
	{
		return notificationInfos;
	}

	public String toString()
	{
		return new StringBuffer("StringMonitor on ").append(super.toString()).toString();
	}
}
