/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package mx4j.adaptor.http;

/**
 * Contains utilities common to CommandProcessor objects
 *
 * @author <a href="mailto:tibu@users.sourceforge.net">Carlos Quiroz</a>
 * @author <a href="mailto:l.dobrev@lsnsoft.com">Lachezar Dobrev</a>
 * @version $Revision: 1.5 $
 */
public class CommandProcessorUtil
{
	/**
	 * Creates a parameter object of the given type containing a given value.
	 * If the type is unknown null is returned
	 *
	 * @param parameterType Indicates the type of the parameter, for instance java.lang.String
	 * @param parameterValue The value of the parameter as a String
	 * @return an object of parameterType type and value parameterValue or null if the type is unknown
	 * @exception Thrown in case there is a data conversion error
	 */
	protected static Object createParameterValue(String parameterType, String parameterValue) throws Exception {
		if (parameterType.equals("java.lang.String"))
		{
			return parameterValue;
		}
		if (parameterType.equals("java.lang.Integer") || parameterType.equals("int"))
		{
			return Integer.valueOf(parameterValue);
		}
		if (parameterType.equals("java.lang.Long") || parameterType.equals("long"))
		{
			return Long.valueOf(parameterValue);
		}
		if (parameterType.equals("java.lang.Short") || parameterType.equals("short"))
		{
			return Short.valueOf(parameterValue);
		}
		if (parameterType.equals("java.lang.Byte") || parameterType.equals("byte"))
		{
			return Byte.valueOf(parameterValue);
		}
		if (parameterType.equals("java.lang.Float") || parameterType.equals("float"))
		{
			return Float.valueOf(parameterValue);
		}
		// changed java.lang.dobule to java.lang.double bronwen
		if (parameterType.equals("java.lang.Double") || parameterType.equals("double"))
		{
				return Double.valueOf(parameterValue);
		}
		if (parameterType.equals("java.lang.Boolean") || parameterType.equals("boolean"))
		{
				return Boolean.valueOf(parameterValue);
		}
		if (parameterType.equals("java.lang.Character") || parameterType.equals("char"))
		{
				if ( parameterValue.length() > 0 )
				{
					return new Character(parameterValue.charAt(0));
				}
				else
				{
					throw new NumberFormatException("Can not initialize Character from empty String");
				}
		}
		Class cls = null;
		java.lang.reflect.Constructor ctor = null;
		try
		{
			cls	= Class.forName(parameterType);
			ctor	= cls.getConstructor(new Class[] {String.class});
			return ctor.newInstance(new Object[] {parameterValue});
		}
		catch (ClassNotFoundException cnfe)
		{
			// Can not find class. Not in our ClassLoader?
			/** @todo Ask the MBeanServer to instantiate this class??? */
			throw new IllegalArgumentException("Invalid parameter type: " + parameterType);
		}
		catch (NoSuchMethodException nsme)
		{
			// No public String constructor.
			throw new IllegalArgumentException("Invalid parameter type: " + parameterType);
		}
		catch (Exception ex)
		{
			// Constructor might have thrown an exception?
			// Security Exception ?
			// IllegalAccessException? .... etc.
			// Just rethrow. We can do little here <shrug>
			/** @todo Log the exception */
			throw ex;
		}
	}

	/**
	 * Checks if the given type is primitive of can be initialized from String.<br>
	 * This is done by trying to load the class and checking if there is a public String
	 * only constructor.
	 *
	 * @param parameterType Indicates the type of the parameter, for instance java.lang.String
	 * @return true if the type is primitive or String initializable
	 * @exception Thrown in case there is a data conversion error
	 */
	protected static boolean canCreateParameterValue(String parameterType){
		if (parameterType.equals("java.lang.String"))
		{
			return true;
		}
		if (parameterType.equals("java.lang.Integer") || parameterType.equals("int"))
		{
			return true;
		}
		if (parameterType.equals("java.lang.Long") || parameterType.equals("long"))
		{
			return true;
		}
		if (parameterType.equals("java.lang.Short") || parameterType.equals("short"))
		{
			return true;
		}
		if (parameterType.equals("java.lang.Byte") || parameterType.equals("byte"))
		{
			return true;
		}
		if (parameterType.equals("java.lang.Float") || parameterType.equals("float"))
		{
			return true;
		}
		if (parameterType.equals("java.lang.Double") || parameterType.equals("double"))
		{
			return true;
		}
		if (parameterType.equals("java.lang.Boolean") || parameterType.equals("boolean"))
		{
			return true;
		}
		if (parameterType.equals("java.lang.Character") || parameterType.equals("char"))
		{
			return true;
		}
		Class cls = null ;
		try
		{
			cls	= Class.forName(parameterType);
			cls.getConstructor(new Class[] {String.class});
			// Yes we can load the class and it has a public String constructor.
			return true;
		}
		catch (ClassNotFoundException cnfe)
		{
			// Can not find class. Not in our ClassLoader?
			/** @todo Ask the MBeanServer to instantiate this class??? */
			return false;
		}
		catch (NoSuchMethodException nsme)
		{
			// No public String constructor.
			return false;
		}
		catch (Exception ex)
		{
			// Constructor might have thrown an exception?
			// Security Exception ?
			// IllegalAccessException? .... etc.
			// Just rethrow. We can do little here <shrug>
			/** @todo Log the exception */
			return false;
		}
	}
}
