/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package mx4j.loading;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.management.ObjectName;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.2 $
 */
public class MLetTag
{
   private String m_code;
   private String m_object;
   private ArrayList m_archives = new ArrayList();
   private String m_codebase;
   private ObjectName m_name;
   private String m_version;
   private ArrayList m_types = new ArrayList();
   private ArrayList m_values = new ArrayList();

   public URL[] getURLs(String defaultCodeBase)
   {
      ArrayList archives = getArchives();
      ArrayList urls = new ArrayList();
      for (int i = 0; i < archives.size(); ++i)
      {
         String archive = (String)archives.get(i);
         URL url = createArchiveURL(defaultCodeBase, archive);
         if (url != null)
         {
            urls.add(url);
         }
      }
      return (URL[])urls.toArray(new URL[urls.size()]);
   }

   private URL createArchiveURL(String defaultCodebase, String archive)
   {
      String codebase = getCodeBase();
      String url = (codebase == null ? defaultCodebase : codebase) + archive;
      try
      {
         return new URL(url);
      }
      catch (MalformedURLException ignored)
      {
         // Maybe it's a file
         File f = new File(url);
         if (f.exists())
         {
            try {return f.toURL();}
            catch (MalformedURLException ignored2) {}
         }
      }
      return null;
   }

   private String getCodeBase()
   {
      return m_codebase;
   }

   private ArrayList getArchives()
   {
      return m_archives;
   }

   public String getCode()
   {
      return m_code;
   }

   public ObjectName getObjectName()
   {
      return m_name;
   }

   public String getObject()
   {
      return m_object;
   }

   public String[] getSignature()
   {
      return m_types == null ? new String[0] : (String[])m_types.toArray(new String[m_types.size()]);
   }

   public Object[] getArguments()
   {
      return m_values == null ? new Object[0] : (Object[])m_values.toArray(new Object[m_values.size()]);
   }

   void setArchive(String archive)
   {
      StringTokenizer tokenizer = new StringTokenizer(archive, ",");
      while (tokenizer.hasMoreTokens())
      {
         String token = tokenizer.nextToken().trim();
         if (token.length() > 0)
         {
            token.replace('\\', '/');
            m_archives.add(token);
         }
      }
   }

   void setCode(String code)
   {
      m_code = code;
   }

   void setCodeBase(String codebase)
   {
      codebase = codebase.replace('\\', '/');
      if (!codebase.endsWith("/")) {codebase += "/";}
      m_codebase = codebase;
   }

   void setName(ObjectName name)
   {
      m_name = name;
   }

   void setObject(String object)
   {
      m_object = object;
   }

   void setVersion(String version)
   {
      m_version = version;
   }

   void addArg(String type, Object value)
   {
      m_types.add(type);
      m_values.add(value);
   }
}
