/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package mx4j.server;

import java.io.ObjectInputStream;
import java.util.Set;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.IntrospectionException;
import javax.management.InvalidAttributeValueException;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.NotCompliantMBeanException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.management.OperationsException;
import javax.management.QueryExp;
import javax.management.ReflectionException;

/**
 * Base class for chained MBeanServer.
 * By default this class delegates all method calls to the nested MBeanServer.
 * Subclass it to add behavior to one or more (or all) methods.
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.2 $
 */
public class ChainedMBeanServer implements MBeanServer
{
	private MBeanServer m_server;


    /**
     * Creates a new ChainedMBeanServer with the specified <code>MBeanServer</code>.
     *
     * @param server The MBeanServer
     */
	public ChainedMBeanServer(MBeanServer server)
	{
		m_server = server;
	}

	protected MBeanServer getMBeanServer()
	{
		return m_server;
	}

    /**
     *
     * @see MBeanServer#addNotificationListener
     */
	public void addNotificationListener(ObjectName observed, NotificationListener listener, NotificationFilter filter, Object handback)
			throws InstanceNotFoundException
	{
		m_server.addNotificationListener(observed, listener, filter, handback);
	}

    
    /**
     *
     * @see MBeanServer#addNotificationListener
     */
	public void addNotificationListener(ObjectName observed, ObjectName listener, NotificationFilter filter, Object handback)
			throws InstanceNotFoundException
	{
		m_server.addNotificationListener(observed, listener, filter, handback);
	}

    /**
     *
     * @see MBeanServer#createMBean
     */
	public ObjectInstance createMBean(String className, ObjectName objectName)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException
	{
		return m_server.createMBean(className, objectName);
	}

    /**
     *
     * @see MBeanServer#createMBean
     */
	public ObjectInstance createMBean(String className, ObjectName objectName, Object[] args, String[] parameters)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException
	{
		return m_server.createMBean(className, objectName, args, parameters);
	}

    /**
     *
     * @see MBeanServer#createMBean
     */
	public ObjectInstance createMBean(String className, ObjectName objectName, ObjectName loaderName)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException
	{
		return m_server.createMBean(className, objectName, loaderName);
	}

    /**
     *
     * @see MBeanServer#createMBean
     */
	public ObjectInstance createMBean(String className, ObjectName objectName, ObjectName loaderName, Object[] args, String[] parameters)
			throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException
	{
		return m_server.createMBean(className, objectName, loaderName, args, parameters);
	}

    /**
     *
     * @see MBeanServer#deserialize
     */
	public ObjectInputStream deserialize(String className, byte[] bytes)
			throws OperationsException, ReflectionException
	{
		return m_server.deserialize(className, bytes);
	}

    /**
     *
     * @see MBeanServer#deserialize
     */
	public ObjectInputStream deserialize(String className, ObjectName loaderName, byte[] bytes)
			throws InstanceNotFoundException, OperationsException, ReflectionException
	{
		return m_server.deserialize(className, loaderName, bytes);
	}

    /**
     *
     * @see MBeanServer#deserialize
     */
	public ObjectInputStream deserialize(ObjectName objectName, byte[] bytes)
			throws InstanceNotFoundException, OperationsException
	{
		return m_server.deserialize(objectName, bytes);
	}

    /**
     *
     * @see MBeanServer#getAttribute
     */
	public Object getAttribute(ObjectName objectName, String attribute)
			throws MBeanException, AttributeNotFoundException, InstanceNotFoundException, ReflectionException
	{
		return m_server.getAttribute(objectName, attribute);
	}

    /**
     *
     * @see MBeanServer#getAttributes
     */
	public AttributeList getAttributes(ObjectName objectName, String[] attributes)
			throws InstanceNotFoundException, ReflectionException
	{
		return m_server.getAttributes(objectName, attributes);
	}

    /**
     *
     * @see MBeanServer#getDefaultDomain
     */
	public String getDefaultDomain()
	{
		return m_server.getDefaultDomain();
	}

    /**
     *
     * @see MBeanServer#getMBeanCount
     */
	public Integer getMBeanCount()
	{
		return m_server.getMBeanCount();
	}

    /**
     *
     * @see MBeanServer#getMBeanInfo
     */
	public MBeanInfo getMBeanInfo(ObjectName objectName)
			throws InstanceNotFoundException, IntrospectionException, ReflectionException
	{
		return m_server.getMBeanInfo(objectName);
	}

    /**
     *
     * @see MBeanServer#getObjectInstance
     */
	public ObjectInstance getObjectInstance(ObjectName objectName)
			throws InstanceNotFoundException
	{
		return m_server.getObjectInstance(objectName);
	}

    /**
     *
     * @see MBeanServer#instantiate
     */
	public Object instantiate(String className)
			throws ReflectionException, MBeanException
	{
		return m_server.instantiate(className);
	}

    /**
     *
     * @see MBeanServer#instantiate
     */
	public Object instantiate(String className, Object[] args, String[] parameters)
			throws ReflectionException, MBeanException
	{
		return m_server.instantiate(className, args, parameters);
	}

    /**
     *
     * @see MBeanServer#instantiate
     */
	public Object instantiate(String className, ObjectName loaderName)
			throws ReflectionException, MBeanException, InstanceNotFoundException
	{
		return m_server.instantiate(className, loaderName);
	}

    /**
     *
     * @see MBeanServer#instantiate
     */
	public Object instantiate(String className, ObjectName loaderName, Object[] args, String[] parameters)
			throws ReflectionException, MBeanException, InstanceNotFoundException
	{
		return m_server.instantiate(className, loaderName, args, parameters);
	}

    /**
     *
     * @see MBeanServer#invoke
     */
	public Object invoke(ObjectName objectName, String methodName, Object[] args, String[] parameters)
			throws InstanceNotFoundException, MBeanException, ReflectionException
	{
		return m_server.invoke(objectName, methodName, args, parameters);
	}

    /**
     *
     * @see MBeanServer#isInstanceOf
     */
	public boolean isInstanceOf(ObjectName objectName, String className)
			throws InstanceNotFoundException
	{
		return m_server.isInstanceOf(objectName, className);
	}

    /**
     *
     * @see MBeanServer#isRegistered
     */
	public boolean isRegistered(ObjectName objectname)
	{
		return m_server.isRegistered(objectname);
	}

    /**
     *
     * @see MBeanServer#queryMBeans
     */
	public Set queryMBeans(ObjectName patternName, QueryExp filter)
	{
		return m_server.queryMBeans(patternName, filter);
	}

    /**
     *
     * @see MBeanServer#queryNames
     */
	public Set queryNames(ObjectName patternName, QueryExp filter)
	{
		return m_server.queryNames(patternName, filter);
	}

    /**
     *
     * @see MBeanServer#registerMBean
     */
	public ObjectInstance registerMBean(Object mbean, ObjectName objectName)
			throws InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException
	{
		return m_server.registerMBean(mbean, objectName);
	}

    /**
     *
     * @see MBeanServer#removeNotificationListener
     */
	public void removeNotificationListener(ObjectName observed, NotificationListener listener)
			throws InstanceNotFoundException, ListenerNotFoundException
	{
		m_server.removeNotificationListener(observed, listener);
	}

    /**
     *
     * @see MBeanServer#removeNotificationListener
     */
	public void removeNotificationListener(ObjectName observed, ObjectName listener)
			throws InstanceNotFoundException, ListenerNotFoundException
	{
		m_server.removeNotificationListener(observed, listener);
	}

    /**
     *
     * @see MBeanServer#setAttribute
     */
	public void setAttribute(ObjectName objectName, Attribute attribute)
			throws InstanceNotFoundException, AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException
	{
		m_server.setAttribute(objectName, attribute);
	}

    /**
     *
     * @see MBeanServer#setAttributes
     */
	public AttributeList setAttributes(ObjectName objectName, AttributeList attributes)
			throws InstanceNotFoundException, ReflectionException
	{
		return m_server.setAttributes(objectName, attributes);
	}

    /**
     *
     * @see MBeanServer#unregisterMBean
     */
	public void unregisterMBean(ObjectName objectName)
			throws InstanceNotFoundException, MBeanRegistrationException
	{
		m_server.unregisterMBean(objectName);
	}
}
