/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package mx4j.server;

import java.util.ArrayList;

import javax.management.loading.MLet;

/**
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.4 $
 */
final class DefaultClassLoaderRepository extends BaseClassLoaderRepository
{
	private ArrayList m_classLoaders = new ArrayList();

	public Class loadClass(String className) throws ClassNotFoundException
	{
		return loadClassWithout(null, className);
	}

	public Class loadClassWithout(ClassLoader loader, String className) throws ClassNotFoundException
	{
		synchronized (m_classLoaders)
		{
			for (int i = 0; i < m_classLoaders.size(); ++i)
			{
				try
				{
					ClassLoader cl = (ClassLoader)m_classLoaders.get(i);
					if (cl.equals(loader)) {continue;}

					// This is an optimization: if the classloader is an MLet (and not a subclass)
					// then the method MLet.loadClass(String, ClassLoaderRepository) is used.
					if (cl.getClass() == MLet.class) return ((MLet)cl).loadClass(className, null);

					return cl.loadClass(className);
				}
				catch (ClassNotFoundException ignored) {}
			}
			throw new ClassNotFoundException(className);
		}
	}

	protected void addClassLoader(ClassLoader cl)
	{
		synchronized (m_classLoaders) {m_classLoaders.add(cl);}
	}

	protected void removeClassLoader(ClassLoader cl)
	{
		synchronized (m_classLoaders) {m_classLoaders.remove(cl);}
	}

	private int getSize()
	{
		synchronized (m_classLoaders) {return m_classLoaders.size();}
	}
}
