/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package mx4j.server;

import java.security.BasicPermission;

/**
 * Permission that MBean class must have in order to be trusted. <p>
 * Only MBeans that have this permission can be registered in the MBeanServer.
 * This permission is composed by a target name, whose only valid value is <code>register</code>.
 * The actions are ignored.
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.3 $
 */
public final class MBeanTrustPermission extends BasicPermission
{
	public MBeanTrustPermission(String name)
	{
		this(name, null);
	}

	public MBeanTrustPermission(String name, String actions)
	{
		super(name, actions);
		if (!"register".equals(name)) {throw new IllegalArgumentException("Target name must be 'register' not " + name);}
	}
}
