/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package mx4j.server.interceptor;

import java.util.List;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.InvalidAttributeValueException;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanRegistrationException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ReflectionException;

import mx4j.server.MBeanMetaData;

/**
 * MBeanServer --> MBean interceptor.
 *
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.3 $
 */
public interface MBeanServerInterceptor
{
	public static final int PRE_REGISTER = 1;
	public static final int POST_REGISTER_TRUE = 2;
	public static final int POST_REGISTER_FALSE = 3;
	public static final int PRE_DEREGISTER = 4;
	public static final int POST_DEREGISTER = 5;

	/**
	 * The type of this interceptor
	 */
	public String getType();

	public void setChain(List interceptors);

	public void addNotificationListener(MBeanMetaData metadata, NotificationListener listener, NotificationFilter filter, Object handback);

	public void removeNotificationListener(MBeanMetaData metadata, NotificationListener listener, NotificationFilter filter, Object handback) throws ListenerNotFoundException;

	public void instantiate(MBeanMetaData metadata, String className, String[] params, Object[] args) throws ReflectionException, MBeanException;

	public void registration(MBeanMetaData metadata, int operation) throws MBeanRegistrationException;

	public MBeanInfo getMBeanInfo(MBeanMetaData metadata);

	public Object invoke(MBeanMetaData metadata, String method, String[] params, Object[] args) throws MBeanException, ReflectionException;

	public AttributeList getAttributes(MBeanMetaData metadata, String[] attributes);

	public AttributeList setAttributes(MBeanMetaData metadata, AttributeList attributes);

	public Object getAttribute(MBeanMetaData metadata, String attribute) throws MBeanException, AttributeNotFoundException, ReflectionException;

	public void setAttribute(MBeanMetaData metadata, Attribute attribute) throws MBeanException, AttributeNotFoundException, InvalidAttributeValueException, ReflectionException;
}
